@@ -1,4 +1,3 @@
-import csv
 import json
 import os
 import pandas as pd
@@ -6,6 +5,7 @@ import pickle
 import sqlite3
 import zipfile
 
+from dask import dataframe as dd
 from sqlalchemy import create_engine
 from typing import List
 
@@ -77,6 +77,7 @@ class DataImporter(FileUtils):
     def __init__(self,
                  file_path: str,
                  as_data_frame: bool = True,
+                 use_dask: bool = True,
                  create_dir: bool = True,
                  sep: str = ',',
                  **kwargs: dict
@@ -88,8 +89,11 @@ class DataImporter(FileUtils):
         :param as_data_frame: bool
             Import data set as pandas data frame or not
 
+        :param use_dask: bool
+            Use dask library for parallel computation
+
         :param create_dir: bool
-            Boolean indicating whether to create directories if they are not existed
+            Create directories if they are not existed
 
         :param sep: str
             File separator
@@ -99,77 +103,102 @@ class DataImporter(FileUtils):
         """
         super().__init__(file_path=file_path, create_dir=create_dir)
         self.as_df: bool = as_data_frame
+        self.use_dask: bool = use_dask
+        self.partitions: int = 4 if kwargs.get('npartitions') is None else kwargs.get('npartitions')
         self.sep: str = sep
-        self.user_kwargs = kwargs
-        self.kwargs = None
+        self.kwargs: dict = kwargs
         self._config_args()
 
     def _config_args(self):
         """
         Set configuration setting for the data import as Pandas DataFrame
         """
-        self.kwargs = {'filepath': self.full_path,
-                       'sep': self.sep,
-                       'decimal': '.',
-                       'header': 0,
-                       'encoding': 'utf-8',
-                       'skip_blank_lines': True,
-                       'na_values': None,
-                       'keep_default_na': True,
-                       'parse_dates': True,
-                       'quotechar': '|',
-                       'quoting': csv.QUOTE_NONE,
-                       'doublequote': False,
-                       'sheet_name': 0,
-                       'names': None,
-                       'index_col': None,
-                       'usecols': None,
-                       'squeeze': False,
-                       'prefix': None,
-                       'mangle_dup_cols': True,
-                       'dtype': None,
-                       'engine': None,
-                       'converters': None,
-                       'true_values': None,
-                       'false_values': None,
-                       'skipinitialspace': False,
-                       'skiprows': None,
-                       'skipfooter': 0,
-                       'nrows': None,
-                       'na_filter': True,
-                       'verbose': False,
-                       'infer_datetime_format': True,
-                       'keep_date_col': False,
-                       'date_parser': None,
-                       'dayfirst': False,
-                       'iterator': False,
-                       'chunksize': None,
-                       'compression': 'infer',
-                       'thousands': None,
-                       'float_precision': None,
-                       'lineterminator': None,
-                       'escapechar': None,
-                       'comment': None,
-                       'dialect': None,
-                       'error_bad_lines': False,
-                       'warn_bad_lines': True,
-                       'low_memory': True,
-                       'memory_map': False
-                       }
-        if self.user_kwargs is not None:
-            for kwarg, value in self.user_kwargs.items():
-                if kwarg in self.kwargs.keys():
-                    self.kwargs[kwarg] = value
-                else:
-                    print('Key word argument ({}) not supported !'.format(kwarg))
+        self.kwargs.update({'filepath': self.full_path,
+                            'sep': self.sep,
+                            'decimal': '.' if self.kwargs.get('decimal') is None else self.kwargs.get('decimal'),
+                            'header': 0 if self.kwargs.get('header') is None else self.kwargs.get('header'),
+                            'encoding': 'utf-8' if self.kwargs.get('encoding') is None else self.kwargs.get('encoding'),
+                            'skip_blank_lines': True if self.kwargs.get('skip_blank_lines') is None else self.kwargs.get('skip_blank_lines'),
+                            'na_values': self.kwargs.get('na_values'),
+                            'keep_default_na': True if self.kwargs.get('keep_default_na') is None else self.kwargs.get('keep_default_na'),
+                            'parse_dates': True if self.kwargs.get('parse_dates') is None else self.kwargs.get('parse_dates'),
+                            'quotechar': '"' if self.kwargs.get('quotechar') is None else self.kwargs.get('quotechar'),
+                            'quoting': 0 if self.kwargs.get('quoting') is None else self.kwargs.get('quotechar'), #csv.QUOTE_NONE,
+                            'doublequote': True if self.kwargs.get('doublequote') is None else self.kwargs.get('doublequote'),
+                            'sheet_name': 0 if self.kwargs.get('sheet_name') is None else self.kwargs.get('sheet_name'),
+                            'names': self.kwargs.get('names'),
+                            'index_col': self.kwargs.get('index_col'),
+                            'usecols': self.kwargs.get('usecols'),
+                            'squeeze': False if self.kwargs.get('squeeze') is None else self.kwargs.get('squeeze'),
+                            'prefix': self.kwargs.get('prefix'),
+                            'mangle_dup_cols': True if self.kwargs.get('mangle_dup_cols') is None else self.kwargs.get('mangle_dup_cols'),
+                            'dtype': self.kwargs.get('dtype'),
+                            'engine': self.kwargs.get('engine'),
+                            'converters': self.kwargs.get('converters'),
+                            'true_values': self.kwargs.get('true_values'),
+                            'false_values': self.kwargs.get('false_values'),
+                            'skipinitialspace': False if self.kwargs.get('skipinitialspace') is None else self.kwargs.get('skipinitialspace'),
+                            'skiprows': self.kwargs.get('skiprows'),
+                            'skipfooter': 0 if self.kwargs.get('skipfooter') is None else self.kwargs.get('skipfooter'),
+                            'nrows': self.kwargs.get('nrows'),
+                            'na_filter': True if self.kwargs.get('na_filter') is None else self.kwargs.get('na_filter'),
+                            'verbose': False if self.kwargs.get('verbose') is None else self.kwargs.get('verbose'),
+                            'infer_datetime_format': True if self.kwargs.get('infer_datetime_format') is None else self.kwargs.get('infer_datetime_format'),
+                            'keep_date_col': False if self.kwargs.get('keep_date_col') is None else self.kwargs.get('keep_date_col'),
+                            'date_parser': None if self.kwargs.get('date_parser') is None else self.kwargs.get('date_parser'),
+                            'dayfirst': False if self.kwargs.get('dayfirst') is None else self.kwargs.get('dayfirst'),
+                            'iterator': False if self.kwargs.get('iterator') is None else self.kwargs.get('iterator'),
+                            'chunksize': self.kwargs.get('chunksize'),
+                            'compression': 'infer' if self.kwargs.get('compression') is None else self.kwargs.get('compression'),
+                            'thousands': self.kwargs.get('thousands'),
+                            'float_precision': self.kwargs.get('float_precision'),
+                            'lineterminator': self.kwargs.get('lineterminator'),
+                            'escapechar': self.kwargs.get('escapechar'),
+                            'comment': self.kwargs.get('comment'),
+                            'dialect': self.kwargs.get('dialect'),
+                            'error_bad_lines': False if self.kwargs.get('error_bad_lines') is None else self.kwargs.get('error_bad_lines'),
+                            'warn_bad_lines': True if self.kwargs.get('warn_bad_lines') is None else self.kwargs.get('warn_bad_lines'),
+                            'low_memory': True if self.kwargs.get('low_memory') is None else self.kwargs.get('low_memory'),
+                            'memory_map': False if self.kwargs.get('memory_map') is None else self.kwargs.get('memory_map')
+                            })
 
-    def _excel_as_df(self) -> pd.DataFrame:
+    def _excel_as_df(self):
         """
         Import excel file as Pandas DataFrame
 
         :return: pd.DataFrame
             Pandas DataFrame containing the content of the html file
         """
+        if self.use_dask:
+            return dd.from_pandas(data=pd.read_excel(io=self.kwargs.get('filepath'),
+                                                     sheet_name=self.kwargs.get('sheet_name'),
+                                                     header=self.kwargs.get('header'),
+                                                     names=self.kwargs.get('names'),
+                                                     index_col=self.kwargs.get('index_col'),
+                                                     usecols=self.kwargs.get('usecols'),
+                                                     squeeze=self.kwargs.get('squeeze'),
+                                                     dtype=self.kwargs.get('dtype'),
+                                                     engine=self.kwargs.get('engine'),
+                                                     converters=self.kwargs.get('converters'),
+                                                     true_values=self.kwargs.get('true_values'),
+                                                     false_values=self.kwargs.get('false_values'),
+                                                     skiprows=self.kwargs.get('skiprows'),
+                                                     skipfooter=self.kwargs.get('skipfooter'),
+                                                     nrows=self.kwargs.get('nrows'),
+                                                     na_values=self.kwargs.get('na_values'),
+                                                     keep_default_na=self.kwargs.get('keep_default_na'),
+                                                     na_filter=self.kwargs.get('na_filter'),
+                                                     verbose=self.kwargs.get('verbose'),
+                                                     parse_dates=self.kwargs.get('parse_dates'),
+                                                     date_parser=self.kwargs.get('date_parser'),
+                                                     thousands=self.kwargs.get('thousands'),
+                                                     comment=self.kwargs.get('comment')
+                                                     ),
+                                  npartitions=self.partitions,
+                                  chunksize=self.kwargs.get('chunksize'),
+                                  sort=True if self.kwargs.get('sort') is None else self.kwargs.get('sort'),
+                                  name=self.kwargs.get('name')
+                                  )
         return pd.read_excel(io=self.kwargs.get('filepath'),
                              sheet_name=self.kwargs.get('sheet_name'),
                              header=self.kwargs.get('header'),
@@ -178,7 +207,6 @@ class DataImporter(FileUtils):
                              usecols=self.kwargs.get('usecols'),
                              squeeze=self.kwargs.get('squeeze'),
                              prefix=self.kwargs.get('prefix'),
-                             #mangle_dupe_cols=self.kwargs.get('mangle_dup_cols'),
                              dtype=self.kwargs.get('dtype'),
                              engine=self.kwargs.get('engine'),
                              converters=self.kwargs.get('converters'),
@@ -199,8 +227,6 @@ class DataImporter(FileUtils):
                              date_parser=self.kwargs.get('date_parser'),
                              dayfirst=self.kwargs.get('dayfirst'),
                              iterator=self.kwargs.get('iterator'),
-                             #chunksize=self.kwargs.get('chunksize'),
-                             #compression=self.kwargs.get('compression'),
                              thousands=self.kwargs.get('thousands'),
                              decimal=self.kwargs.get('decimal'),
                              float_precision=self.kwargs.get('float_precision'),
@@ -214,7 +240,6 @@ class DataImporter(FileUtils):
                              error_bad_lines=self.kwargs.get('error_bad_lines'),
                              warn_bad_lines=self.kwargs.get('warn_bad_lines'),
                              low_memory=self.kwargs.get('low_memory'),
-                             #memory_map=self.kwargs.get('memory_map')
                              )
 
     def _file(self):
@@ -281,27 +306,41 @@ class DataImporter(FileUtils):
                              object_pairs_hook=self.kwargs.get('object_pairs_hook')
                              )
 
-    def _json_as_df(self) -> pd.DataFrame:
+    def _json_as_df(self):
         """
         Import json file as Pandas DataFrame
 
-        :return: pd.DataFrame
+        :return: Pandas DataFrame or dask dataframe
             Content of the json file
         """
+        if self.use_dask:
+            return dd.read_json(url_path=self.kwargs.get('filepath'),
+                                orient='records' if self.kwargs.get('orient') is None else self.kwargs.get('orient'),
+                                lines=self.kwargs.get('lines'),
+                                storage_options=self.kwargs.get('storage_options'),
+                                blocksize=self.kwargs.get('blocksize'),
+                                sample=2 ** 20 if self.kwargs.get('sample') is None else self.kwargs.get('sample'),
+                                encoding='utf-8' if self.kwargs.get('encoding') is None else self.kwargs.get('encoding'),
+                                errors='strict' if self.kwargs.get('errors') is None else self.kwargs.get('errors'),
+                                compression='infer' if self.kwargs.get('compression') is None else self.kwargs.get('compression'),
+                                meta=self.kwargs.get('meta'),
+                                engine=pd.read_json
+                                )
         return pd.read_json(path_or_buf=self.kwargs.get('filepath'),
-                            orient=None,
+                            orient='records' if self.kwargs.get('orient') is None else self.kwargs.get('orient'),
                             typ='frame',
-                            dtype=True,
-                            convert_axes=True,
-                            convert_dates=True,
-                            keep_default_dates=True,
-                            numpy=False,
-                            precise_float=False,
-                            date_unit=None,
-                            encoding=None,
-                            lines=False,
-                            chunksize=None,
-                            compression=self.kwargs.get('compression'))
+                            dtype=True if self.kwargs.get('dtype') is None else self.kwargs.get('dtype'),
+                            convert_axes=True if self.kwargs.get('convert_axes') is None else self.kwargs.get('convert_axes'),
+                            convert_dates=True if self.kwargs.get('convert_dates') is None else self.kwargs.get('convert_dates'),
+                            keep_default_dates=True if self.kwargs.get('keep_default_dates') is None else self.kwargs.get('keep_default_dates'),
+                            numpy=False if self.kwargs.get('numpy') is None else self.kwargs.get('numpy'),
+                            precise_float=False if self.kwargs.get('precise_float') is None else self.kwargs.get('precise_float'),
+                            date_unit=self.kwargs.get('date_unit'),
+                            encoding='utf-8' if self.kwargs.get('encoding') is None else self.kwargs.get('encoding'),
+                            lines=False if self.kwargs.get('lines') is None else self.kwargs.get('lines'),
+                            chunksize=self.kwargs.get('chunksize'),
+                            compression=self.kwargs.get('compression')
+                            )
 
     def _pickle(self) -> pickle.load:
         """
@@ -313,22 +352,40 @@ class DataImporter(FileUtils):
         with open(self.file_path, 'rb') as file:
             return pickle.load(file=file)
 
-    def _pickle_as_df(self) -> pd.DataFrame:
+    def _pickle_as_df(self):
         """
         Import pickle file as Pandas DataFrame
 
-        :return: pd.DataFrame
+        :return: Pandas DataFrame or dask dataframe
             Content of the pickle file
         """
-        return pd.read_pickle(path=self.full_path, compression=self.kwargs.get('compression'))
+        if self.use_dask:
+            return dd.from_pandas(data=pd.read_pickle(filepath_or_buffer=self.full_path, compression=self.kwargs.get('compression')),
+                                  npartitions=self.partitions,
+                                  chunksize=self.kwargs.get('chunksize'),
+                                  sort=True if self.kwargs.get('sort') is None else self.kwargs.get('sort'),
+                                  name=self.kwargs.get('name')
+                                  )
+        return pd.read_pickle(filepath_or_buffer=self.full_path, compression=self.kwargs.get('compression'))
 
-    def _text_as_df(self) -> pd.DataFrame:
+    def _text_as_df(self):
         """
         Import text file (csv, txt) as Pandas DataFrame
 
-        :return: pd.DataFrame
+        :return: Pandas DataFrame or dask dataframe
             Content of the text file
         """
+        if self.use_dask:
+            return dd.read_csv(urlpath=self.kwargs.get('filepath'),
+                               blocksize='default' if self.kwargs.get('blocksize') is None else self.kwargs.get('blocksize'),
+                               lineterminator=self.kwargs.get('lineterminator'),
+                               #compression=self.kwargs.get('compression'),
+                               sample=256000 if self.kwargs.get('sample') is None else self.kwargs.get('sample'),
+                               enforce=False if self.kwargs.get('enforce') is None else self.kwargs.get('enforce'),
+                               assume_missing=True if self.kwargs.get('assume_missing') is None else self.kwargs.get('assume_missing'),
+                               storage_options=self.kwargs.get('storage_options'),
+                               include_path_column=False if self.kwargs.get('include_path_column') is None else self.kwargs.get('include_path_column')
+                               )
         return pd.read_csv(filepath_or_buffer=self.kwargs.get('filepath'),
                            sep=self.kwargs.get('sep'),
                            header=self.kwargs.get('header'),
@@ -376,10 +433,13 @@ class DataImporter(FileUtils):
                            memory_map=self.kwargs.get('memory_map')
                            )
 
-    def file(self):
+    def file(self, table_name: str = None):
         """
         Import data from file
 
+        :param table_name: str
+            Name of the table of the local database file
+
         :return: object
             File content
         """
@@ -393,6 +453,21 @@ class DataImporter(FileUtils):
             return self._html_as_df() if self.as_df else self._file()
         elif self.file_type in ['xls', 'xlsx']:
             return self._excel_as_df() if self.as_df else self._file()
+        elif self.file_type == 'db':
+            _con = DBUtils(table_name=table_name, file_path=self.full_path)
+            _con.create_connection()
+            try:
+                if self.use_dask:
+                    _df: dd.DataFrame = dd.from_pandas(data=_con.get_table(), npartitions=self.partitions)
+                    #_df: dd.DataFrame = dd.read_sql_table(table=table_name, uri=self.full_path, index_col='', divisions=self.partitions)
+                else:
+                    _df: pd.DataFrame = _con.get_table()
+            except sqlite3.Error as e:
+                _df: pd.DataFrame = pd.DataFrame()
+                print(e)
+            finally:
+                _con.con.close()
+            return _df
         else:
             raise FileUtilsException('File type ({}) not supported'.format(self.file_type))
 
@@ -569,7 +644,6 @@ class DBUtils:
 
         :param file_path
             File path of the local SQLite3 database
-
         """
         self.df = df
         self.table_name = table_name
